import { Button } from 'antd';
import React, { useState, useRef, useEffect } from 'react';
import Expression from './Expression';
type Props = {
    value?: any,
    onChange?: (v: any) => void
}
const EditablePre: React.FC<Props> = (props) => {
    const { value, onChange } = props
    const editableDivRef = useRef<HTMLDivElement>(null);
    const [savedRange, setSavedRange] = useState<Range | null>(null); // 用于存储光标位置
    //内容变更
    const handleInput = () => {
        console.log(11111)
        if (editableDivRef.current) {
            const newStr = editableDivRef.current.innerHTML
                .replace(/<div[^>]*>(\s|&nbsp;)*<\/div>/g, '') // 移除空的 <div> 标签
                .replace(/<div[^>]*>/g, '') // 将 <div> 替换为 ""
                .replace(/<\/div>/g, '\n') // 将 </div> 替换为 "\n"
                .replace(/<br\s*\/?>/g, '\n') // 替换 <br> 为换行符
            console.log(newStr)
            onChange?.(newStr);
        }
    };
    // 记录光标位置
    const handleFocusLoss = () => {
        const selection: any = window.getSelection();
        if (selection?.rangeCount > 0) {
            setSavedRange(selection.getRangeAt(0)); // 记录当前光标位置
        }
    };
    const insertAtSavedPosition = (html: string) => {
        if (!savedRange) return; // 如果没有记录的光标位置，则不执行
    
        const tempDiv = document.createElement('div');
        tempDiv.innerHTML = html; // 创建包含 HTML 的临时元素
        const frag = document.createDocumentFragment();
    
        // 将临时元素的子元素添加到文档片段中
        while (tempDiv.firstChild) {
            frag.appendChild(tempDiv.firstChild);
        }
    
        // 插入内容到记录的光标位置
        savedRange.deleteContents(); // 删除光标处的内容
        savedRange.insertNode(frag); // 将内容插入光标位置
    
        // 重新设置光标位置在插入内容之后
        if (frag.lastChild) {
            // 设置光标在插入节点后
            const selection:any = window.getSelection();
            selection.removeAllRanges();
            savedRange.setStartAfter(frag.lastChild); // 设置光标在插入节点后
            savedRange.collapse(true); // 折叠到新的光标位置
            selection.addRange(savedRange); // 将新的范围添加到选区
        }
    };
    const insertHtmlAtCursor = (html: string) => {
        const selection = window.getSelection();
        if (!selection || selection.rangeCount === 0) return;

        const range = selection.getRangeAt(0);
        range.deleteContents(); // 删除光标处的内容

        const tempDiv = document.createElement('div');
        tempDiv.innerHTML = html; // 创建包含 HTML 的临时元素
        const frag = document.createDocumentFragment();

        // 将临时元素的子元素添加到文档片段中
        while (tempDiv.firstChild) {
            frag.appendChild(tempDiv.firstChild);
        }
        range.insertNode(frag); // 将内容插入光标位置

        // 重新设置光标位置
        range.collapse(false);
        selection.removeAllRanges();
        selection.addRange(range);
        handleInput()
    };

    useEffect(() => {
        if (value) {
            let newHtml = value?.split('\n')
                .map((line: any, index: any) => `<div>${line}</div>`)
                .join('');
            if (editableDivRef.current) {
                editableDivRef.current.innerHTML = newHtml.replace(/<div[^>]*>(\s|&nbsp;)*<\/div>/g, '')
            }
        }
    }, [])
    return (
        <div>
            <div
                ref={editableDivRef}
                contentEditable
                onInput={handleInput}
                onBlur={handleFocusLoss}
                style={{
                    border: '1px solid #ccc',
                    padding: '10px',
                    minHeight: '100px',
                    overflowY: 'auto',
                    whiteSpace: 'pre-wrap',
                    outline: 'none',
                }}
            >
                {/* 初始内容可以在此处放置 */}
            </div>
            <Expression addEmo={insertAtSavedPosition} range={savedRange as Range} />
            <Button onClick={() => insertHtmlAtCursor('<a href="https://example.com" target="_blank">https://example.com</a>')}>插入链接</Button>
            <Button onClick={() => insertHtmlAtCursor('@用户昵称')}>@用户昵称</Button>
        </div>
    );
};

export default EditablePre;