import { PlusOutlined } from "@ant-design/icons"
import { Button, Drawer, Tag } from "antd"
import Book from '@/pages/MiniApp/BookManage/index'
import { useEffect, useState } from "react"
import styles from './drawer.less'
import { useAjax } from "@/Hook/useAjax"
import useApi from "@/Hook/useApi"
import { useModel } from "@umijs/max"
import { MyIcon } from "@/global"
function BookSelectDrawer(props: { value?: any, onChange?: (v: any) => void }) {
    let [open, setOpen] = useState(false)
    let { initialState } = useModel("@@initialState")
    let apiobj = useApi(initialState?.selectApp?.appCategory || 1)
    let getList = useAjax((params) => apiobj.getBookPageList(params))
    useEffect(() => {
        if (!props?.value?.bookId && props?.value) {
            getList.run({
                pageSize: 20,
                pageNum: 1,
                appId: initialState?.selectApp?.id || "",
                appType: initialState?.selectApp?.appType || "",
                bookId: props?.value
            }).then(res => {
                if (res?.data?.total > 0) {
                    props.onChange?.(res.data.records[0]?.shortBookInfoVO ? { ...res.data.records[0]?.shortBookInfoVO, ...res.data.records[0] } : { ...res.data.records[0]?.longBookInfo, ...res.data.records[0] })
                }
            })
        }
    }, [props?.value])
    return <div>
        <div>
            {!props?.value ? <Button type="dashed" onClick={() => { setOpen(true) }} className={styles.addBtn}>
                <PlusOutlined style={{ fontSize: 20 }} />
            </Button> :
                <div onClick={() => { setOpen(true) }} className={styles.book}>
                    <span>
                        {props?.value?.vipFree && <MyIcon type="icon-vipmianfei" style={{ fontSize: 35, position: 'absolute', zIndex: 1 }} />}
                        <img className={styles.addBtn} src={props?.value?.picUrl} onError={(e: any) => {
                            e.target.src = localStorage.getItem("nocover")
                        }} />
                    </span>
                    <div>
                        <strong>{props?.value?.bookName || "书名"}</strong>
                        <span>{props?.value?.bookDesc || "描述"}</span>
                        <div>
                            <span>{props?.value?.authorInfo?.authorName}·{props?.value?.wordCount}万字·{props?.value?.bookStatus == 0 ? "连载中" : "完结"}</span>
                            <Tag >{props?.value?.labelInfoList?.[0]?.name}</Tag>
                        </div>
                    </div>
                </div>
            }
        </div>
        <Drawer
            title={"小说选择"}
            open={open}
            footer={false}
            width={"80%"}
            onClose={() => { setOpen(false) }}
            destroyOnClose
        >
            <Book {...props} closeDrawer={setOpen} />
        </Drawer>
    </div>
}
export default BookSelectDrawer